library api_cache_manager;

import 'models/cache_db_model.dart';
import 'utils/cache_db_helper.dart';

class APICacheManager {
  static APICacheManager? _instance;

  factory APICacheManager() {
    if (_instance == null) {
      _instance = new APICacheManager._();
    }
    return _instance!;
  }

  APICacheManager._();

  Future<bool> addCacheData(APICacheDBModel model) async {
    await APICacheDBHelper.init();
    bool isSaved = false;
    model.syncTime = new DateTime.now().millisecondsSinceEpoch;

    int res;

    if (!await this.isAPICacheKeyExist(model.key)) {
      res = await APICacheDBHelper.insert(APICacheDBModel.table, model);
    } else {
      res = await APICacheDBHelper.customUpdate(
        APICacheDBModel.table,
        model,
        columnName: "key",
        columnValue: model.key,
      );
    }

    isSaved = res > 0 ? true : false;

    return isSaved;
  }

  Future<APICacheDBModel> getCacheData(String key) async {
    await APICacheDBHelper.init();

    List<Map<String, dynamic>> cacheData =
        await APICacheDBHelper.conditionalQuery(
      APICacheDBModel.table,
      "key = ?",
      key,
    );

    return cacheData.map((item) => APICacheDBModel.fromMap(item)).first;
  }

  Future<bool> isAPICacheKeyExist(String key) async {
    await APICacheDBHelper.init();

    List<Map<String, dynamic>> cacheData =
        await APICacheDBHelper.conditionalQuery(
      APICacheDBModel.table,
      "key = ?",
      key,
    );

    return cacheData.length == 1 ? true : false;
  }

  Future<void> emptyCache() async {
    await APICacheDBHelper.init();
    APICacheDBHelper.deleteAll(APICacheDBModel.table);
  }
}
